@@ -13,6 +13,8 @@ import types
 import io
 import numpy as np
 from uuid import UUID
+import cv2
+import base64
 
 
 class NumpyEncoder(json.JSONEncoder):
@@ -144,6 +146,54 @@ class FetchComponent(Component):
         )
 
 
+class SlideshowComponent(FetchComponent):
+    rest_verb = ["GET"]
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    def dj_query_route(self):
+        fetch_metadata = self.fetch_metadata
+
+        # Dj query provided should return only a video location
+        video_name = (fetch_metadata["query"] & self.restriction).fetch1(
+            *fetch_metadata["fetch_args"]
+        )
+        video = cv2.VideoCapture(video_name)
+
+        payload_size = 0  # bytes
+        encoded_frames = []
+        last_chunk = False
+
+        chunk_size = int(request.args["chunk_size"])
+        start_frame = int(request.args["start_frame"])
+
+        video.set(cv2.CAP_PROP_POS_FRAMES, start_frame)
+        while payload_size < chunk_size:
+            success, frame = video.read()
+            if not success:
+                last_chunk = True
+                break
+            encoded_f = cv2.imencode(".jpeg", frame)[1].tobytes()
+            encoded_frames.append(base64.b64encode(encoded_f).decode())
+            payload_size += 1
+
+        return (
+            NumpyEncoder.dumps(
+                {
+                    "frameMeta": {
+                        "fps": 50,
+                        "frameCount": len(encoded_frames),
+                        "finalChunk": last_chunk,
+                    },
+                    "frames": encoded_frames,
+                }
+            ),
+            200,
+            {"Content-Type": "application/json"},
+        )
+
+
 class DeleteComponent(Component):
     rest_verb = ["DELETE"]
 
@@ -157,11 +207,14 @@ class DeleteComponent(Component):
 class InsertComponent(Component):
     rest_verb = ["POST", "GET"]
     fields_route_format = "{route}/fields"
+    presets_route_format = "{route}/presets"
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        component_config = kwargs.get("component_config", args[1] if args else None)
-        self.fields_map = component_config.get("map")
+        self.component_config = kwargs.get(
+            "component_config", args[1] if args else None
+        )
+        self.fields_map = self.component_config.get("map")
         self.tables = [
             getattr(
                 dj.VirtualModule(
@@ -172,7 +225,8 @@ class InsertComponent(Component):
                 t,
             )
             for s, t in (
-                _.format(**request.args).split(".") for _ in component_config["tables"]
+                _.format(**request.args).split(".")
+                for _ in self.component_config["tables"]
             )
         ]
         self.parents = sorted(
@@ -193,6 +247,24 @@ class InsertComponent(Component):
         }
         self.input_lookup = {v: k for k, v in self.destination_lookup.items()}
 
+        if "presets" in self.component_config:
+            lcls = locals()
+            exec(self.component_config["presets"], globals(), lcls)
+            self.presets = lcls["presets"]
+
+            self.preset_vm_list = [
+                dj.VirtualModule(
+                    s,
+                    s.replace("__", "-"),
+                    connection=self.connection,
+                )
+                for s in inspect.getfullargspec(self.presets).args
+            ]
+
+    @property
+    def presets_dict(self):
+        return self.presets(*self.preset_vm_list)
+
     def dj_query_route(self):
         with self.connection.transaction:
             for t in self.tables:
@@ -214,7 +286,7 @@ class InsertComponent(Component):
         source_fields = {
             **{
                 (p_name := f"{p.database}.{dj.utils.to_camel_case(p.table_name)}"): {
-                    "values": p.fetch("KEY"),
+                    "values": [NumpyEncoder.dumps(row) for row in p.fetch("KEY")],
                     "type": "table",
                     "name": p_name,
                 }
@@ -247,7 +319,12 @@ class InsertComponent(Component):
                     **(
                         {
                             "values": [
-                                {self.input_lookup.get(k, k): v for k, v in r.items()}
+                                json.dumps(
+                                    {
+                                        self.input_lookup.get(k, k): v
+                                        for k, v in json.loads(r).items()
+                                    }
+                                )
                                 for r in field["values"]
                             ]
                         }
@@ -260,6 +337,45 @@ class InsertComponent(Component):
             + list(source_fields.values())
         )
 
+    def presets_route(self):
+        # Table content for presets should follow the following format:
+        #
+        # preset_names: string
+        # ---
+        # presets: blob or json
+        #
+        # Example result from query:
+        # [['preset_name', {"b_id": 1, "b_number": 2345}],
+        # ['preset2_name', {"b_id": 13, "b_number": 225}]]
+        #
+        # If you have a name mapping it will be applied to each preset
+        # Route will 404 if no preset query is defined and 500 if there is an Exception
+
+        # Helper function to filter out fields not in the insert,
+        # as well as apply the fields_map
+        def filterPreset(preset: dict):
+            return {
+                (self.input_lookup[k] if k in self.input_lookup else k): v
+                for k, v in preset.items()
+            }
+
+        if "presets" not in self.component_config:
+            return (
+                "No Preset query found",
+                404,
+                {"Content-Type": "text/plain"},
+            )
+
+        filtered_preset_dictionary = {
+            k: filterPreset(v) for k, v in self.presets_dict.items()
+        }
+
+        return (
+            NumpyEncoder.dumps(filtered_preset_dictionary),
+            200,
+            {"Content-Type": "application/json"},
+        )
+
 
 class TableComponent(FetchComponent):
     attributes_route_format = "{route}/attributes"
@@ -466,5 +582,6 @@ type_map = {
     "slider": FetchComponent,
     "dropdown-query": FetchComponent,
     "form": InsertComponent,
+    "slideshow": SlideshowComponent,
     "delete": DeleteComponent,
 }
