@@ -1,5 +1,7 @@
 """Exposed REST API."""
 from os import environ
+from pathlib import Path
+from envyaml import EnvYAML
 from .interface import _DJConnector
 import datajoint as dj
 from . import __version__ as version
@@ -126,7 +128,7 @@ def api_version() -> str:
             Content-Type: application/json
 
             {
-                "version": "0.7.2"
+                "version": "0.7.3"
             }
 
         :statuscode 200: No error.
@@ -1063,6 +1065,32 @@ def attribute(
             return traceback.format_exc(), 500
 
 
+@app.route(
+    f"{environ.get('PHARUS_PREFIX', '')}/spec",
+    methods=["GET"],
+)
+@protected_route
+def spec(
+    connection: dj.Connection,
+) -> dict:
+    # Returns the currently loaded spec sheet
+    if request.method in {"GET"}:
+        try:
+            try:
+                from .getSpecOverride import get_spec
+            except ImportError:
+
+                def get_spec(connection):
+                    spec_path = environ.get("PHARUS_SPEC_PATH")
+                    return EnvYAML(Path(spec_path))["SciViz"]
+
+            except Exception as e:
+                raise e
+            return get_spec(connection)
+        except Exception:
+            return traceback.format_exc(), 500
+
+
 @app.route(
     f"{environ.get('PHARUS_PREFIX', '')}/schema/<schema_name>/table/<table_name>/dependency",
     methods=["GET"],
